@@ -41,6 +41,31 @@ class Dataset(object):
                 label_dict = os.path.join(package_root, 'labeltemplate',
                                           'aasm.labeltemplate'),
                 mode = 'memory'):
+        '''
+        Create a new `Dataset()`
+
+        Parameters
+        ----------
+        dataset_name : str
+            The name of the dataset. It will be the filename if you save this
+            dataset.
+        save_path : path-like
+            Where you want to save this dataset.
+        comment : str, optional
+            A comment string to describe your dataset. The default is ''.
+        label_dict : path-like, optional
+            A dictionary to manage labels of dataset. It will translate human-
+            readable label to the style which fits machine learning. The
+            default is the AASM protocal.
+        mode : {'memory', 'disk'}, optional
+            For the faster speed, 'memory' mode will process all the data in
+            the memory. But if you have a huge dataset which exceeds or will
+            exhaust your memory, the 'disk' mode will cache all the data at
+            disk until you are to use them. It do slow down the computation,
+            but it is a good resolution when you are not able to extend your
+            memory. The default is 'memory'.
+
+        '''
         self.VERSION = 0.2
         self.path = save_path
         if self.path is not None and (dataset_name is not None):
@@ -501,14 +526,15 @@ class Dataset(object):
     def __df_del_epoch(self, disk_file, data_name, epoch):
         disk_file['dataset'][data_name]['data'].pop(str(epoch))
 
-    def select_epochs(self, label_name=None, label_value=None, data_name=None):
+    def select_epochs(self, element_name=None, element_value=None,
+                      data_name=None):
         # data_name
         if data_name is None:
             data_name = self.get_data()
         elif isinstance(data_name, str):
             data_name = [data_name]
         # label_name & label_value
-        if (label_name is None) and (label_value is None):
+        if element_name is None:
             rst = list()
             for d in data_name:
                 if self.__get_state(d) < Dataset.PREPROCESSED:
@@ -516,21 +542,27 @@ class Dataset(object):
                 logging.info(d)
                 for epoch in self.get_epochs(d):
                     rst.append((d, epoch))
-        elif label_name is not None and label_value is not None:
+        elif element_name is not None and element_value is not None:
             rst = list()
-            if not isinstance(label_value, list):
-                label_value = [label_value]
+            if not isinstance(element_value, list):
+                element_value = [element_value]
             for d in data_name:
                 if self.__get_state(d) == Dataset.ERROR:
                     continue
                 logging.info(d)
-                for epoch in self.get_epochs(d):
-                    for l in label_value:
-                        if self.get_label(d, epoch, label_name) == l:
+                if self.elements[element_name] == 'label':
+                    for epoch in self.get_epochs(d):
+                        if self.get_label(d, epoch, element_name) in \
+                            element_value:
+                            rst.append((d, epoch))
+                elif self.elements[element_name] == 'condition':
+                    if self.get_condition(d, element_name) in element_value:
+                        # if matches the condition, add all the epochs into rst
+                        for epoch in self.get_epochs(d):
                             rst.append((d, epoch))
         else:
             raise TypeError('You should input all of or none of\n' + 
-                            '`label_type` and `label_value`')
+                            '`element_name` and `element_value`')
         return rst
 
     def exclude_epochs(self, label_name = None, label_value = None):
@@ -1191,7 +1223,7 @@ class Dataset(object):
 
     ### SAMPLE ###
 
-    def stat_classes(self, elements):
+    def stat_classes(self, elements, unit=None):
         '''
         Add up different classes (or class group) according to input elements.
         Return a dict whose `keys()` are classes and `values()` are epochs for
@@ -1205,9 +1237,25 @@ class Dataset(object):
 
         Parameters
         ----------
-        elements : str or list
-            `element_name` or a list of them. Must be `label_name` or
-            `condition_type`.
+        elements : str, list or dict
+            `element_name` or a list of them. Also can be a dict with format:
+            >>> {element_name : [element_value, element_value]}
+            to add up specific classes.
+
+            For example, you want to add up classes whose `'LABEL'` is '1' or
+            '2', and `'DIAGNOSE'` is `'healthy'`. You can use:
+            >>> dataset.stat_classes(
+            ...    {'LABEL' : ['1', '2'],
+            ...     'DIAGNOSE' : 'healthy'})
+            
+            If you want to add up all the `'LABEL'` but just when `'DIAGNOSE'`
+            is `'healthy'`. You can use:
+            >>> dataset.stat_classes(
+            ...    {'LABEL' : [],  # an empty list means use all the values
+            ...     'DIAGNOSE' : 'healthy'})
+            Notice that an empty list means use all the values.
+
+            Elements must be `label_name` or `condition_type`.
 
         Raises
         ------
@@ -1226,15 +1274,24 @@ class Dataset(object):
             `(data_name, epoch)` tuples.
 
         '''
-        
+        # check input
+        select = False  # to control whether to use selected elements or all
+        unit_candidate = ['epoch', 'data']
+        if unit is not None and unit not in unit_candidate:
+            raise ValueError('Invalid unit. Only \'epoch\' or \'data\' allowd')
         if isinstance(elements, str):
             elements = [elements]
         elif isinstance(elements, list):
             pass
+        elif isinstance(elements, dict):
+            select = True
+            for key in elements:
+                if isinstance(elements[key], str):
+                    elements[key] = [elements[key]]
         else:
-            raise TypeError('The input parameter `elements` must be `str` or\n'
-                            '`list`. It represents label names or condition\n'
-                            'types.')
+            raise TypeError('The input parameter `elements` must be `str`, '
+                            '`list` or `dict`. It represents label names or '
+                            'condition types.')
 
         num = len(elements)  # numbers of elements
 
@@ -1254,11 +1311,24 @@ class Dataset(object):
                                  ' names or condition types.')
         # check passed
         rst = dict()
-        if len(label) == 0:  # the unit is `data`
-            for d in self.get_data():
+        if unit is None:
+            if len(label) == 0:
+                unit = 'data'
+            else:
+                unit = 'epoch'
+
+        if unit == 'data':
+            for d in self.get_data():  # traversal and add up classes
+                fit = True
                 c = []
                 for cond in condition:
+                    if select and elements[cond] and self.get_condition(d, cond) \
+                        not in elements[cond]:  # encounter an invalid condition
+                        fit = False
+                        break  # drop this data
                     c.append(self.get_condition(d, cond))
+                if not fit:
+                    continue
                 c = tuple(c) if num > 1 else c[0]  # use tuple or single
                                                    # element
                 if c not in rst:
@@ -1267,11 +1337,22 @@ class Dataset(object):
         else:  # the unit is `epoch`
             for d in self.get_data():
                 for e in self.get_epochs(d):
+                    fit = True
                     c = []
                     for cond in condition:
+                        if select and elements[cond] and self.get_condition(d, \
+                            cond) not in elements[cond]:  # encounter an invalid condition
+                            fit = False
+                            break  # drop this epoch
                         c.append(self.get_condition(d, cond))
                     for l in label:
+                        if select and elements[l] and self.get_label(d, e, l) \
+                            not in elements[l]:  # encounter an invalid label
+                            fit = False
+                            break  # drop this data
                         c.append(self.get_label(d, e, l))
+                    if not fit:
+                        continue
                     c = tuple(c) if num > 1 else c[0]  # use tuple or single 
                                                        # element
                     if c not in rst:
@@ -1281,7 +1362,7 @@ class Dataset(object):
 
     def sample_data(self, data_name, lst, tmin=0, tmax=0, data_padding=True,
                     max_len=None, epoch_padding=False, test_data_name=None,
-                    autoencoder=False):
+                    autoencoder=False, array_type=int):
         if test_data_name is None:
             test_data_name = data_name
 
@@ -1308,8 +1389,9 @@ class Dataset(object):
                 y = [y]
             for i in y:
                 y_samp.append(
-                    self.label_dict[i].get_array( \
-                        self.get_condition(test_data_name, i)))
+                    self.label_dict[i].get_array(
+                        self.get_condition(test_data_name, i),
+                        array_type=array_type))
             if len(y_samp) == 1:
                 y_samp = y_samp[0]
             else:
@@ -1318,7 +1400,7 @@ class Dataset(object):
 
     def sample_epoch(self, data_name, epoch, lst, tmin=0, tmax=0,
                     epoch_padding=False, test_data_name=None, test_epoch=None,
-                    autoencoder=False):
+                    autoencoder=False, array_type=int):
         if test_data_name is None:
             test_data_name = data_name
             test_epoch = epoch
@@ -1329,7 +1411,8 @@ class Dataset(object):
             x = [x]
         for i in x:
             x_samp.append(self.sample_epoched_x(data_name, epoch, i,
-                                                tmin, tmax, epoch_padding))
+                                                tmin, tmax, epoch_padding,
+                                                array_type=array_type))
         if len(x_samp) == 1:
             x_samp = x_samp[0]
         else:
@@ -1345,103 +1428,83 @@ class Dataset(object):
                 y = [y]
             for i in y:
                 y_samp.append(self.sample_epoched_y(test_data_name, test_epoch,
-                                                    i))
+                                                    i, array_type=array_type))
             if len(y_samp) == 1:
                 y_samp = y_samp[0]
             else:
                 y_samp = np.asarray(y_samp)
             return (x_samp, y_samp)
 
-    """
-    def sample_epoched_x(self, data_name, epoch, element_name, tmin=0,
-                               tmax=0, padding=False, array_type=int):
-        '''当返回None时，跳过这个epoch'''
-        # check state
-        if self.__get_state(data_name) < Dataset.PREPROCESSED:
-            raise DataStateError('You cannot sample from a data not correctly '
-                                 'preprocessed. The target data `' + data_name
-                                 + '` has a state `' 
-                                 + self.__get_state(data_name, True) + '`.')
-        # without timestep
-        if tmin == 0 and tmax == 0:
-            # feature
-            if self.elements[element_name] == 'feature':
-                return self.get_feature(data_name, epoch, element_name)
-            # label
-            elif self.elements[element_name] == 'label':
-                return self.get_label(data_name, epoch, element_name)
-        # with timestep
-        else:
-            # check timespan
-            # timespan cannot be longer than data_length
-            data_length = len(self.get_epochs(data_name))
-            logging.info('Data Length: ' + str(data_length))
-            assert tmax >= 0
-            assert abs(tmin) + tmax <= data_length
-            # sample
-            epoch_list = self.get_epochs(data_name)
-            # at the start of the series
-            if epoch_list[epoch] < epoch_list[abs(tmin)]:
-                if padding:  # pre-padding
-                    # feature
-                    if self.elements[element_name] == 'feature':
-                        r = np.array([np.zeros(self.shape[element_name])] \
-                                     * (abs(tmin) - epoch) 
-                            + [self.get_feature(data_name, i, element_name) \
-                               for i in epoch_list[: epoch + tmax + 1]])
-                    # label
-                    elif self.elements[element_name] == 'label':
-                        p = self.label_dict[element_name].get_array( \
-                                                None, array_type = array_type)
-                        r = np.array([p] * (abs(tmin) - epoch) 
-                            + [self.label_dict[element_name].get_array( \
-                            self.get_label(data_name,
-                                             epoch[i], element_name), \
-                                array_type = array_type) \
-                               for i in epoch_list[: epoch + tmax + 1]])
-                # if not padding, the epoch at edge will be disposed
-                else:
-                    raise BrokenTimestepError()
-            # at the end of the series
-            elif epoch_list[epoch] > epoch_list[data_length - tmax - 1]:
-                if padding:  # post-padding
-                    # feature
-                    if self.elements[element_name] == 'feature':
-                        r = np.array([self.get_feature(data_name,
-                                                         i, element_name) \
-                                      for i in epoch_list[epoch - abs(tmin) :]]
-                        + [np.zeros(self.shape[element_name])] \
-                            * (tmax + epoch + 1 - data_length))
-                    # label
-                    elif self.elements[element_name] == 'label':
-                        p = self.label_dict[element_name].get_array( \
-                                                None, array_type = array_type)
-                        r = np.array([self.label_dict[element_name].get_array(
-                                        self.get_label(data_name,
-                                                         i, element_name), \
-                                            array_type = array_type) \
-                                      for i in epoch_list[epoch - abs(tmin) :]]
-                        + [p] * (tmax + epoch + 1 - data_length))
-            else: # normal situation
-                # feature
-                if self.elements[element_name] == 'feature':
-                    r = np.array([self.get_feature(data_name, i,
-                                                     element_name) 
-                                  for i in epoch_list[epoch - abs(tmin) : \
-                                                      epoch + tmax + 1]])
-                # label
-                elif self.elements[element_name] == 'label':
-                    r = np.array([self.label_dict[element_name].get_array( \
-                            self.get_label(data_name, epoch[i], element_name),
-                            array_type = array_type)
-                            for i in epoch_list[epoch - abs(tmin) : \
-                                                      epoch + tmax + 1]])
-            return r
-    """
+    def sample_epoched_x(self, data_name:str, epoch:int, element_name:str,
+                         tmin:int=0, tmax:int=0, padding:bool=False,
+                         array_type:type=int, concat:bool=False):
+        '''
+        A low-level method to sample one epoch according to parameters.
 
-    def sample_epoched_x(self, data_name, epoch, element_name, tmin=0,
-                               tmax=0, padding=False, array_type=int):
-        '''当返回None时，跳过这个epoch'''
+        Parameters
+        ----------
+        data_name : str
+            The target datum.
+        epoch : int
+            The target epoch.
+        element_name : str
+            The target element, it may be a feature_name, a label_name or a
+            condition_name.
+        tmin : int, optional
+            The number of preceding epochs before the target epoch to be
+            sampled. Usually used for RNN-related model or other models related
+            to time series. The default is 0, means do not sample any other
+            epochs.
+        tmax : int, optional
+            The number of following epochs after the target epoch to be
+            sampled. Usually used for RNN-related model or other models related
+            to time series. The default is 0, means do not sample any other
+            epochs.
+        padding : bool, optional
+            At the beginning or end of the datum, whether the target epoch to
+            be padded with zero or dropped. If True, they will be padded, 
+            otherwise dropped. The default is False.
+        array_type : type or None, optional
+            The type of elements in return array. The default and most
+            recommended is int. You should input directly a `type`, not a
+            `str`.
+            For example:
+                array_type=bool          (Correct)
+                array_type='bool'        (Wrong)
+
+        concat : bool, optional
+            Decide the output shape when tmin!=0 or tmax!=0, i.e., multiple
+            time steps are sampled.
+            If you have not set the `tmin` or `tmax`. You do not need to care
+            about this parameter.
+            The default is False, means the output will not concatenate time
+            steps (i.e. sampled epochs) but leave the output shape as
+            `(time_step, channel[, feature-related])`. In this situation, every
+            time step occupies a unique matrix. It is suitable for RNN-related
+            models.
+            If `concat==True`, all the sampled epoches will concatenate by the
+            'time' axis. It results in continuous feature which is suitable for
+            TCN or other CNN-related models.
+
+        Raises
+        ------
+        DataStateError
+            Raised when your data have not preprocessed yet. Check your data
+            state.
+        BrokenTimestepError
+            The flag of the end of one datum. It is for the iterators. You need
+            not do any treatment about it.
+        ValueError
+            The shape mismatch.
+
+        Returns
+        -------
+        np.ndarray
+            The sampled x.
+
+        '''
+        # Skip this epoch if returns `None`
+        logging.info('== EPOCHED SAMPLE ==')
         # check state
         if self.__get_state(data_name) < Dataset.PREPROCESSED:
             raise DataStateError('You cannot sample from a data not correctly '
@@ -1451,15 +1514,17 @@ class Dataset(object):
                                                            data_name, True)))
         logging.debug('Data_name: {0}'.format(data_name))
         logging.debug('Epoch: {0}'.format(str(epoch)))
-        # without timestep
+        # ===== without timestep =====
         if tmin == 0 and tmax == 0:
-            # feature
+            # == feature ==
             if self.elements[element_name] == 'feature':
                 return self.get_feature(data_name, epoch, element_name)
-            # label
+            # == label ==
             elif self.elements[element_name] == 'label':
-                return self.get_label(data_name, epoch, element_name)
-        # with timestep
+                return self.label_dict[element_name].get_array( \
+                        self.get_label(data_name, epoch, element_name),
+                        array_type=array_type)
+        # ===== with timestep =====
         else:
             # check timespan
             # timespan cannot be longer than data_length
@@ -1477,19 +1542,19 @@ class Dataset(object):
             # so check if `idx - abs(tmin) < 0`
             lower = idx - abs(tmin) if idx - abs(tmin) >= 0 else 0
             upper = idx + tmax + 1
-            # feature
+            # == feature ==
             if self.elements[element_name] == 'feature':
                 dtype = 'float32'
                 sample_shape = self.shape[element_name]
                 r = np.asarray([self.get_feature(data_name, i, element_name) 
                               for i in epoch_list[lower : upper]])
-            # label
+            # == label ==
             elif self.elements[element_name] == 'label':
                 dtype = 'int32'
                 sample_shape = self.label_dict[element_name].shape()
                 r = np.asarray([self.label_dict[element_name].get_array( \
                         self.get_label(data_name, i, element_name),
-                        array_type = array_type)
+                        array_type=array_type)
                         for i in epoch_list[lower : upper]])
             if len(r) < timespan:
                 if not padding:
@@ -1508,6 +1573,10 @@ class Dataset(object):
                     elif idx - abs(tmin) < 0:
                         x[-len(trunc):] = trunc
                     r = x
+            # == concat ==
+            # Only when `x` is a feature, the concat is needed.
+            if concat and self.elements[element_name] == 'feature':
+                r = np.concatenate(r)
             logging.debug(r.shape)
             return r
 
@@ -1522,26 +1591,27 @@ class Dataset(object):
         if self.elements[element_name] == 'label':
             return self.label_dict[element_name].get_array( \
                         self.get_label(data_name, epoch, element_name),
-                        array_type = array_type)
+                        array_type=array_type)
         elif self.elements[element_name] == 'condition':
             return self.label_dict[element_name].get_array( \
                         self.get_condition(data_name, 
                                            condition_type=element_name),
-                        array_type = array_type)
+                        array_type=array_type)
         else:
             raise ValueError('The input element ' + element_name + 'is a `'
                              + self.elements[element_name] + '`. It cannot'
                              'act as `epoched_y` sample.')
     
     def sample_serial_x(self, data_name, element_name, tmin, tmax,
-                        data_padding, max_len, epoch_padding):
+                        data_padding, max_len, epoch_padding, array_type=int):
         timespan = abs(tmin) + tmax + 1
         rst = []
         for epoch in self.get_epochs(data_name):
             try:
                 rst.append(self.sample_epoched_x(data_name, epoch, 
                                                  element_name,
-                                                 tmin, tmax, epoch_padding))
+                                                 tmin, tmax, epoch_padding,
+                                                 array_type=array_type))
             except BrokenTimestepError:
                 continue
         if data_padding:
@@ -1683,6 +1753,28 @@ class Dataset(object):
 
     @staticmethod
     def load(path, mode = 'memory', load_source = False):
+        '''
+        Load the existed dataset from disk.
+
+        Parameters
+        ----------
+        path : path-like
+            The path to the dataset file.
+        mode : {'memory', 'disk'}, optional
+            For the faster speed, 'memory' mode will process all the data in
+            the memory. But if you have a huge dataset which exceeds or will
+            exhaust your memory, the 'disk' mode will cache all the data at
+            disk until you are to use them. It do slow down the computation,
+            but it is a good resolution when you are not able to extend your
+            memory. The default is 'memory'.
+        load_source : bool, optional
+            Whether to load the sources of dataset. The default is False.
+
+        Returns
+        -------
+        dataset : sleepology.dataset.Dataset
+
+        '''
         # 2020-3-1 hdf5->dict
         disk_file = h5py.File(path, 'r')
         # 获取属性
@@ -1708,10 +1800,4 @@ class Dataset(object):
         self.__df_save_attr_0_2(disk_file)
         self.__df_save_label_dict_0_2(disk_file)
         self.__df_save_dataset_0_2(disk_file)
-        disk_file.close()
-
-
-class IndexManager(object):
-    def __init__(self, *args):
-        self.idx = {}
-        
\ No newline at end of file
+        disk_file.close()
\ No newline at end of file
