@@ -5,13 +5,14 @@ Created on Wed Aug 26 20:07:43 2020
 @author: 赵匡是
 """
 from .exceptions import ModeError, LackOfParameterError, BrokenTimestepError,\
-    KFoldError, LackOfLabelDictError
+    KFoldError, LackOfLabelDictError, TaskError
 
+import math
 import json
+import random
 import logging
 import numpy as np
 import tableprint as tp
-from random import shuffle as sfl
 from sklearn.model_selection import ShuffleSplit
 
 def flatten(seq):
@@ -33,6 +34,7 @@ class Sample(object):
         self.set_epoch_padding(epoch_padding)
         self.set_data_padding(data_padding)
         self.set_task(task)
+        self.array_type = int
 
     def set_unit(self, unit):
         '''
@@ -171,6 +173,39 @@ class Sample(object):
         else:
             self.task = task.lower()
 
+    def set_label_array_type(self, array_type):
+        '''
+        Set the type of one-hot or N-hot label array the sample will return.
+        If `array_type` is `None`, the function will return a number directly,
+        i.e., not in terms of one-hot or N-hot array.
+
+        Parameters
+        ----------
+        array_type : type or None, optional
+            The type of elements in return array. The default and most
+            recommended is int. You should input directly a `type`, not a
+            `str`.
+            For example:
+                array_type=bool          (Correct)
+                array_type='bool'        (Wrong)
+
+            If `array_type` is `None`, the function will return a number
+            directly, i.e., not in terms of one-hot or N-hot array.
+
+        Raises
+        ------
+        TypeError
+            Raised when you input an empty `label` parameter.
+
+        Returns
+        -------
+        int or np.ndarray
+            return `int` if array_type is None.
+            else return one-hot or N-hot `np.ndarray`. 
+
+        '''
+        self.array_type = array_type
+
     def set_x(self, x):
         # 不强求类型。可以是dataset的label甚至是condition(比如用很多condition来判断疾病(age, sex)→diagnose
         self.__check_editable()
@@ -226,7 +261,11 @@ class Sample(object):
                    ['x', str(self.get_x())],
                    ['y', str(self.get_y())],
                    ['Sample Mode', self.mode if hasattr(self, 'mode') else 
-                    'Not Set']]
+                    'Not Set'],
+                   ['label_array_type', '{0}: {1}'.format(
+                       'one-hot/N-hot' if self.array_type else 'numeric',
+                       self.array_type if self.array_type else 'int'
+                   )]]
         tp.banner('Your Sampling Settings')
         tp.table(content, headers)
 
@@ -298,6 +337,14 @@ class Sample(object):
 
     def __check_label_dict(self, dataset):
         if not self.__autoencoder:
+            for x in self.get_x():
+                if dataset.elements[x] == 'label' and x not in \
+                    dataset.label_dict:
+                    raise LackOfLabelDictError('The element `{0}` does not '
+                                               'have a label_dict. Please use '
+                                               '`Dataset.one_hot()` function '
+                                               'to create one or manually set '
+                                               'one.'.format(x))
             for y in self.get_y():
                 if y not in dataset.label_dict:
                     raise LackOfLabelDictError('The element `{0}` does not '
@@ -309,7 +356,7 @@ class Sample(object):
     def __check_iteration(self):
         '''
         In `train` mode, before use `Sample.train_set()` or `Sample.test_set()`
-        , one should first get one fold (or a subset for crossvalidation).
+        , we should get one fold first (or a subset for cross validation).
         Unless the cross validation is not needed (when `n_splits == 1`).
         
         If `n_splits != 1`, you should use `Sample.next_fold()` every iteration
@@ -388,7 +435,49 @@ class Sample(object):
                                                  'cannot represent a data)')
         # and in prediction mode, `y` will not be used
 
-    def from_dataset(self, dataset, data_selection = None, mode = 'train'):
+    def from_dataset(self, dataset, data_selection:dict=None,
+                     mode:str='train'):
+        '''
+        Bind your sampling plan with a `Dataset`, which enables you begin to
+        sample, but also disallows you to change your sampling plan again.
+
+        Parameters
+        ----------
+        dataset : sleepology.dataset.Dataset
+            A `Dataset` object. You should first create a `Dataset` from source
+            or load a `Dataset` from disk, then input the `Dataset` as a
+            parameter.
+
+        data_selection : dict, optional
+            The data or epochs you want to sample. The default is None which
+            means use all the data or epochs.
+            You should input a dict whose keys are `element_name` and values
+            are `element_value`.
+            For example, you have a `Dataset`, it will be sampled in unit of
+            epochs. And it has `PSG` as an element of feature, and `LABEL` as
+            an element of label, and also `DIAGNOSE` as an element of
+            condition. You just want to sample epochs whose `LABEL` is `1` or 
+            '2' and `DIAGNOSE` is `'healthy'`, you can use:
+            >>> data_selection = {'LABEL' : [1, 2].
+            ...                   'DIAGNOSE' : ['healthy']}
+
+        mode : {'train', 'predict'}, optional
+            Define the purpose why you sample. If you want to train a model,
+            the sampled data will be balanced and split into train set and
+            test set. Else you want to use an existed model to predict new
+            data, the `Sample` process should not do any change to data, and it
+            will definitely no `Sample.y()` because the correct answer has not
+            provided. The default is 'train'.
+
+        Raises
+        ------
+        ValueError
+
+        Returns
+        -------
+        None.
+
+        '''
         # set mode
         try:
             if mode.lower() == 'train':
@@ -413,13 +502,44 @@ class Sample(object):
         # set dataset
         self.dataset = dataset
         self.dataset.shape_check()
-        if data_selection is None and self.unit == 'epoch':
+        if data_selection is None and self.get_unit() == 'epoch':
+            self.__selection = None
             self.data_selection = self.dataset.select_epochs()
-        elif data_selection is None and self.unit == 'data':
+        elif data_selection is None and self.get_unit() == 'data':
+            self.__selection = None
             self.data_selection = self.dataset.select_data()
         else:
-            self.data_selection = data_selection
-
+            # data_selection has parameter in form of dict
+            self.__selection = data_selection
+            self.data_selection = set()
+            if self.get_unit() == 'epoch':
+                for e in data_selection.keys():
+                    if self.data_selection and len(data_selection[e]) != 0:
+                        self.data_selection.intersection_update(
+                            self.dataset.select_epochs(e, data_selection[e]))
+                    elif (not self.data_selection) and \
+                        len(data_selection[e]) != 0:
+                        self.data_selection = \
+                            self.dataset.select_epochs(e, data_selection[e])
+                    elif self.data_selection:
+                        pass
+                    else:
+                        self.data_selection = \
+                            self.dataset.select_epochs()
+            else:
+                for e in data_selection.keys():
+                    if self.data_selection and len(data_selection[e]) != 0:
+                        self.data_selection.intersection_update(
+                            self.dataset.select_data(e, data_selection[e]))
+                    elif (not self.data_selection) and \
+                        len(data_selection[e]) != 0:
+                        self.data_selection = \
+                            self.dataset.select_data(e, data_selection[e])
+                    elif self.data_selection:
+                        pass
+                    else:
+                        self.data_selection = \
+                            self.dataset.select_data()
         # set max_len
         if self.data_padding and self.max_len is None:
             epochs = self.dataset.epochs_per_data(self.data_list())
@@ -468,8 +588,18 @@ class Sample(object):
         Split data or epochs by classes into subgroups.
         Generate `Sample.classes`, which is a `dict` of different classes.
         '''
+                # this method only supports `classification` task, not `regression`
+        # task. Here we check it.
+        if self.task != 'classification':
+            raise TaskError('Only `classification` task has subgroups. Please '
+                            'check the returned table when doing '
+                            '`Sample.from_dataset`.')
         logging.info('== DISCRIMINATE DIFFERENT CLASSES ==')
-        classes = self.dataset.stat_classes(self.get_y())
+        if self.__selection:
+            element = self.__selection
+        else:
+            element = self.get_y()
+        classes = self.dataset.stat_classes(element, self.get_unit())
         self.classes = {}
         for c in classes:
             self.classes[c] = [i for i in classes[c] 
@@ -631,16 +761,16 @@ class Sample(object):
         # finally
         if shuffle:
             logging.info('shuffle...')
-            sfl(train_set)
-            sfl(test_set)
+            random.shuffle(train_set)
+            random.shuffle(test_set)
         self.train = train_set
         self.test = test_set
         # disturb, which disturb the match of x and y, i.e. shuffle the y
         if disturb:
             train_set = train_set.copy()
             test_set = test_set.copy()
-            sfl(train_set)
-            sfl(test_set)
+            random.shuffle(train_set)
+            random.shuffle(test_set)
             self.train_y = train_set
             self.test_y = test_set
         logging.info('Length of `train_set` this fold: {0}'.format(
@@ -686,7 +816,22 @@ class Sample(object):
             rst[t[0]].append(t)
         return rst
 
-    def train_set(self):
+    def train_set(self, generator=True):
+        if generator:
+            logging.info('Use generator')
+            return self.train_set_generator()
+        else:
+            logging.info('Do not use generator')
+            x_samp = []
+            y_samp = []
+            for x, y in self.train_set_generator():
+                x_samp.append(x)
+                y_samp.append(y)
+            x_samp = np.asarray(x_samp)
+            y_samp = np.asarray(y_samp)
+            return (x_samp, y_samp)
+
+    def train_set_generator(self):
         self.__check_mode('train')
         self.__check_iteration()
         for idx, item in enumerate(self.train):
@@ -699,7 +844,8 @@ class Sample(object):
                         tmin=self.get_tmin(),
                         tmax=self.get_tmax(), 
                         epoch_padding=self.epoch_padding,
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'data' and not self.disturb:
                     yield self.dataset.sample_data(
                         item,
@@ -709,7 +855,8 @@ class Sample(object):
                         data_padding=self.data_padding,
                         max_len=self.max_len,
                         epoch_padding=self.epoch_padding,
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'epoch' and self.disturb:
                     yield self.dataset.sample_epoch(
                         item[0],
@@ -720,7 +867,8 @@ class Sample(object):
                         epoch_padding=self.epoch_padding,
                         test_data_name=self.train_y[idx][0],
                         test_epoch=self.train_y[idx][1],
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'data' and not self.disturb:
                     yield self.dataset.sample_data(
                         item,
@@ -731,11 +879,27 @@ class Sample(object):
                         max_len=self.max_len,
                         epoch_padding=self.epoch_padding,
                         test_data_name=self.train_y[idx],
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
             except BrokenTimestepError:
                 continue
 
-    def test_set(self):
+    def test_set(self, generator=True):
+        if generator:
+            logging.info('Use generator')
+            return self.test_set_generator()
+        else:
+            logging.info('Do not use generator')
+            x_samp = []
+            y_samp = []
+            for x, y in self.test_set_generator():
+                x_samp.append(x)
+                y_samp.append(y)
+            x_samp = np.asarray(x_samp)
+            y_samp = np.asarray(y_samp)
+            return (x_samp, y_samp)
+
+    def test_set_generator(self):
         self.__check_mode('train')
         self.__check_iteration()
         for idx, item in enumerate(self.test):
@@ -748,7 +912,8 @@ class Sample(object):
                         tmin=self.get_tmin(),
                         tmax=self.get_tmax(), 
                         epoch_padding=self.epoch_padding,
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'data' and not self.disturb:
                     yield self.dataset.sample_data(
                         item,
@@ -758,7 +923,8 @@ class Sample(object):
                         data_padding=self.data_padding,
                         max_len=self.max_len,
                         epoch_padding=self.epoch_padding,
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'epoch' and self.disturb:
                     yield self.dataset.sample_epoch(
                         item[0],
@@ -769,7 +935,8 @@ class Sample(object):
                         epoch_padding=self.epoch_padding,
                         test_data_name=self.test_y[idx][0],
                         test_epoch=self.test_y[idx][1],
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
                 elif self.get_unit() == 'data' and self.disturb:
                     yield self.dataset.sample_data(
                         item,
@@ -780,10 +947,182 @@ class Sample(object):
                         max_len=self.max_len,
                         epoch_padding=self.epoch_padding,
                         test_data_name=self.test_y[idx],
-                        autoencoder=self.__autoencoder)
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
+            except BrokenTimestepError:
+                continue
+
+    def one(self, generator:bool=True):
+        '''
+        Sample only one data and its label in dataset. This one data will
+        be repeatedly sampled and fed into the model. This method is designed
+        for testing a model. If the model cannot fit just one sample, there
+        maybe some problems with the model, such as gradient disappearance.
+
+        Parameters
+        ----------
+        generator : bool, optional
+            Determine the return type. If True, the returned object is a
+            generator, it is supported by tensorflow and memory-saving. If
+            False, the returned object is a np.ndarray, it is supported by
+            scikit-learn.
+            The default is True.
+
+        Returns
+        -------
+        generator or np.ndarray
+
+        '''
+        if generator:
+            logging.info('Use generator')
+            return self.one_generator()
+        else:
+            logging.info('Do not use generator')
+            x_samp = []
+            y_samp = []
+            for x, y in self.one_generator():
+                x_samp.append(x)
+                y_samp.append(y)
+            x_samp = np.asarray(x_samp)
+            y_samp = np.asarray(y_samp)
+            return (x_samp, y_samp)
+
+    def one_generator(self):
+        '''
+        Sample only one data and its label in dataset. This one data will
+        be repeatedly sampled and fed into the model. This method is designed
+        for testing a model. If the model cannot fit just one sample, there
+        maybe some problems with the model, such as gradient disappearance.
+
+        Yields
+        ------
+        np.ndarray
+
+        '''
+        # the purpose of `Sample.one` is only to get one sample, regardless of
+        # whether the mode `train` or `test`, or whether the dataset needs to
+        # cross validation or not.
+        one = random.sample(self.data_selection, 1)
+        one = one * len(self.data_selection)
+        for idx, item in enumerate(one):
+            try:
+                if self.get_unit() == 'epoch':
+                    yield self.dataset.sample_epoch(
+                        item[0],
+                        item[1],
+                        (self.get_x(), self.get_y()),
+                        tmin=self.get_tmin(),
+                        tmax=self.get_tmax(), 
+                        epoch_padding=self.epoch_padding,
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
+                elif self.get_unit() == 'data':
+                    yield self.dataset.sample_data(
+                        item,
+                        (self.get_x(), self.get_y()),
+                        tmin=self.get_tmin(),
+                        tmax=self.get_tmax(),
+                        data_padding=self.data_padding,
+                        max_len=self.max_len,
+                        epoch_padding=self.epoch_padding,
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
+            except BrokenTimestepError:
+                continue
+    
+    def one_per_class(self, generator=True):
+        '''
+        Sample only one data and its label per class. These data will be
+        repeatedly sampled and fed into the model. This method is designed
+        for testing a model. If the model cannot fit such small dataset, there
+        maybe some problems with the model, such as gradient disappearance.
+
+        Parameters
+        ----------
+        generator : bool, optional
+            Determine the return type. If True, the returned object is a
+            generator, it is supported by tensorflow and memory-saving. If
+            False, the returned object is a np.ndarray, it is supported by
+            scikit-learn.
+            The default is True.
+
+        Returns
+        -------
+        generator or np.ndarray
+
+        '''
+        # this method only supports `classification` task, not `regression`
+        # task. Here we check it.
+        if self.task != 'classification':
+            raise TaskError('Only `classification` task has classes. Please '
+                            'check the returned table when doing '
+                            '`Sample.from_dataset` if your task is '
+                            'classification, or use `Sample.one()` instead '
+                            'of `Sample.one_per_class()` in regression task,')
+        if generator:
+            logging.info('Use generator')
+            return self.one_per_class_generator()
+        else:
+            logging.info('Do not use generator')
+            x_samp = []
+            y_samp = []
+            for x, y in self.one_per_class_generator():
+                x_samp.append(x)
+                y_samp.append(y)
+            x_samp = np.asarray(x_samp)
+            y_samp = np.asarray(y_samp)
+            return (x_samp, y_samp)
+
+    def one_per_class_generator(self):
+        '''
+        Sample only one data and its label per class. These data will be
+        repeatedly sampled and fed into the model. This method is designed
+        for testing a model. If the model cannot fit such small dataset, there
+        maybe some problems with the model, such as gradient disappearance.
+
+        Yields
+        ------
+        np.ndarray
+
+        '''
+        if not hasattr(self, 'classes'):
+            self.subgroups()
+        # the purpose of `Sample.one_per_class` is only to get samples,
+        # regardless of whether the mode `train` or `test`, or whether the
+        # dataset needs to cross validation or not.
+        opc = [] # one_per_class
+        for k in self.classes.keys():
+            opc.extend(random.sample(self.classes[k], 1))
+        # the length of samples = data_length
+        opc = opc * math.floor(
+            len(self.data_selection) / len(self.classes.keys()))
+        for idx, item in enumerate(opc):
+            try:
+                if self.get_unit() == 'epoch':
+                    yield self.dataset.sample_epoch(
+                        item[0],
+                        item[1],
+                        (self.get_x(), self.get_y()),
+                        tmin=self.get_tmin(),
+                        tmax=self.get_tmax(), 
+                        epoch_padding=self.epoch_padding,
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
+                elif self.get_unit() == 'data':
+                    yield self.dataset.sample_data(
+                        item,
+                        (self.get_x(), self.get_y()),
+                        tmin=self.get_tmin(),
+                        tmax=self.get_tmax(),
+                        data_padding=self.data_padding,
+                        max_len=self.max_len,
+                        epoch_padding=self.epoch_padding,
+                        autoencoder=self.__autoencoder,
+                        array_type=self.array_type)
             except BrokenTimestepError:
                 continue
 
     def sample(self):
         self.__check_mode('predict')
-        pass
\ No newline at end of file
+        pass
+    
\ No newline at end of file
